@@ -41,8 +41,10 @@
 #include <gio/gio.h>
 
 #include "update-notifier.h"
+#include "livepatch-tray.h"
 #include "update.h"
 #include "hooks.h"
+#include "reboot.h"
 #include "uevent.h"
 #include "crash.h"
 #include "release.h"
@@ -159,20 +161,23 @@ void invoke(const gchar *cmd, const gchar *desktop, gboolean with_pkexec)
       return;
    }
 
-   // fake window to get the current server time *urgh*
-   if (!w) {
-      w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-      gtk_widget_realize (w);
-   }
-
    // normal launch
    context = gdk_display_get_app_launch_context (gdk_display_get_default ());
-   guint32 timestamp =  gdk_x11_get_server_time (gtk_widget_get_window (w));
+   if (GDK_IS_X11_DISPLAY (gdk_display_get_default ())) {
+      // fake window to get the current server time *urgh*
+      if (!w) {
+         w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+         gtk_widget_realize (w);
+      }
+
+      guint32 timestamp =  gdk_x11_get_server_time (gtk_widget_get_window (w));
+      gdk_app_launch_context_set_timestamp (context, timestamp);
+   }
+
    appinfo = g_app_info_create_from_commandline(cmd, 
 						cmd, 
 						G_APP_INFO_CREATE_NONE,
 						&error);
-   gdk_app_launch_context_set_timestamp (context, timestamp);
    if (!g_app_info_launch (appinfo, NULL, (GAppLaunchContext*)context, &error))
       g_warning ("Launching failed: %s", error->message);
    g_object_unref (context);
@@ -188,17 +193,7 @@ invoke_with_pkexec(const gchar *cmd)
         argv[0] = "/usr/bin/pkexec";
         argv[1] = (gchar*)cmd;
         argv[2] = NULL;
-        g_spawn_async (NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, NULL, NULL);
-}
-
-void
-avahi_disabled()
-{
-        g_debug("avahi disabled ()");
-        gchar *argv[2];
-        argv[0] = "/usr/lib/update-notifier/local-avahi-notification";
-        argv[1] = NULL;
-        g_spawn_async (NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, NULL, NULL);
+        g_spawn_async (NULL, argv, NULL, G_SPAWN_DEFAULT, NULL, NULL, NULL, NULL);
 }
 
 void livepatch_check()
@@ -207,7 +202,7 @@ void livepatch_check()
         gchar *argv[2];
         argv[0] = "/usr/lib/update-notifier/livepatch-notification";
         argv[1] = NULL;
-        g_spawn_async (NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, NULL, NULL);
+        g_spawn_async (NULL, argv, NULL, G_SPAWN_DEFAULT, NULL, NULL, NULL, NULL);
 }
 
 static gboolean
@@ -260,11 +255,19 @@ monitor_cb(GFileMonitor *handle,
       g_debug_inotify("apt_get_running=TRUE");
       un->apt_get_running=TRUE;
    }
+   if (g_str_has_prefix(info_uri, DPKG_INFO_DIR)) {
+      g_debug_inotify("/var/lib/dpkg/info/ updated");
+      dpkg_info_updated();
+   }
    if(strstr(info_uri, "/var/lib/update-notifier/dpkg-run-stamp") ||
       strstr(info_uri, "/var/lib/apt/periodic/update-success-stamp")) {
       g_debug_inotify("dpkg_was_run=TRUE");
       un->dpkg_was_run = TRUE;
    }
+   if(strstr(info_uri, REBOOT_FILE)) {
+      g_debug_inotify("reboot required");
+      un->reboot_pending = TRUE;
+   }
    if(strstr(info_uri, HOOKS_DIR)) {
       g_debug_inotify("new hook!");
       un->hook_pending = TRUE;
@@ -273,10 +276,6 @@ monitor_cb(GFileMonitor *handle,
       g_debug_inotify("crashreport found");
       un->crashreport_pending = TRUE;
    }
-   if(strstr(info_uri, UNICAST_LOCAL_AVAHI_FILE)) {
-      g_debug_inotify("avahi disabled due to unicast .local domain");
-      un->unicast_local_avahi_pending = TRUE;
-   }
    if(strstr(info_uri, LIVEPATCH_FILE)) {
       g_debug_inotify("livepatch status changed");
       un->livepatch_pending = TRUE;
@@ -290,8 +289,8 @@ monitor_cb(GFileMonitor *handle,
  * - apt_get_running: set when apt/dpkg activity is detected (in the
  *                    lists-dir, archive-cache, or /var/lib/dpkg/status)
  * - hook_pending: we have new upgrade hook information
+ * - reboot_pending: we need to reboot
  * - crashreport_pending: we have a new crashreport
- * - unicast_local_avahi_pending: avahi got disabled due to a unicast .local domain
  * - livepatch_pending: livepatch status changed
  *
  */
@@ -330,6 +329,12 @@ file_monitor_periodic_check(gpointer data)
       if(un->update_finished_timer > 0) 
 	 g_source_remove(un->update_finished_timer);
 
+      if(un->reboot_pending) {
+	 //g_print("checking reboot now\n");
+	 reboot_check (un->reboot);
+	 un->reboot_pending = FALSE;
+      }
+
       // apt must be finished when a new stamp file was written, so we
       // reset the apt_get_running time-slice field because its not
       // important anymore (it finished running)
@@ -371,12 +376,6 @@ file_monitor_periodic_check(gpointer data)
       un->crashreport_pending = FALSE;
    }
 
-   if(un->unicast_local_avahi_pending) {
-      g_debug("checking for disabled avahi due to unicast .local domain now");
-      avahi_disabled ();
-      un->unicast_local_avahi_pending = FALSE;
-   }
-
   if(un->livepatch_pending) {
       g_debug("checking for livepatch status now");
       livepatch_check ();
@@ -406,6 +405,7 @@ monitor_init(UpgradeNotifier *un)
       "/var/cache/apt/archives/", "/var/cache/apt/archives/partial/",
       HOOKS_DIR,
       CRASHREPORT_DIR,
+      DPKG_INFO_DIR,
       NULL};
    for(i=0;monitor_dirs[i] != NULL;i++) {
       if (getenv("UPSTART_SESSION") && monitor_dirs[i] == CRASHREPORT_DIR) {
@@ -425,13 +425,10 @@ monitor_init(UpgradeNotifier *un)
       "/var/lib/dpkg/status",
       "/var/lib/update-notifier/dpkg-run-stamp",
       "/var/lib/apt/periodic/update-success-stamp",
-      UNICAST_LOCAL_AVAHI_FILE,
       LIVEPATCH_FILE,
+      REBOOT_FILE,
       NULL};
    for(i=0;monitor_files[i] != NULL;i++) {
-      if (getenv("UPSTART_SESSION") && monitor_files[i] == UNICAST_LOCAL_AVAHI_FILE) {
-         continue;
-      }
       GError *error = NULL;
       GFile *gf = g_file_new_for_path(monitor_files[i]);
       monitor_handle = g_file_monitor_file(gf, 0, NULL, &error);
@@ -470,11 +467,21 @@ tray_icons_init(UpgradeNotifier *un)
    trayapplet_create(un->hook, un, "hook-notifier");
    hook_tray_icon_init(un->hook);
 
+   /* reboot required icon */
+   un->reboot = g_new0 (TrayApplet, 1);
+   trayapplet_create(un->reboot, un, "un-reboot");
+   reboot_tray_icon_init(un->reboot);
+
    /* crashreport detected icon */
    un->crashreport = g_new0 (TrayApplet, 1);
    trayapplet_create(un->crashreport, un, "apport");
    crashreport_tray_icon_init(un->crashreport);
 
+   /* livepatch icon */
+   un->livepatch = g_new0(TrayApplet, 1);
+   trayapplet_create(un->livepatch, un, "livepatch");
+   livepatch_tray_icon_init(un->livepatch);
+
    return FALSE; // for the tray_destroyed_cb
 }
 
@@ -549,6 +556,32 @@ sigint_cb (gpointer user_data __attribute__ ((__unused__)))
 	return FALSE;
 }
 
+static time_t
+dpkg_info_time_since_update_internal(gboolean reset)
+{
+    static time_t last_update = 0;
+    time_t current_time;
+
+    current_time = time(NULL);
+
+    if (reset || last_update == 0)
+        last_update = current_time;
+
+    return current_time - last_update;
+}
+
+time_t
+dpkg_info_time_since_update(void)
+{
+    return dpkg_info_time_since_update_internal(/* reset = */FALSE);
+}
+
+void
+dpkg_info_updated(void)
+{
+    (void) dpkg_info_time_since_update_internal(/* reset = */TRUE);
+}
+
 
 static GOptionEntry entries[] =
 {
@@ -654,6 +687,11 @@ main (int argc, char **argv)
 #ifdef ENABLE_SCP
 	uevent_init();
 #endif
+
+	/* check if livepatch patches has been applied during boot or before
+	   update-notifier has started (see LP: #1800862) */
+	livepatch_check ();
+
 	// init gio file monitoring
 	monitor_init (un);
 
