@@ -522,9 +522,6 @@ auto_launch_now (TrayApplet *ta)
    if (auto_launch_security_now(priv, now, last_launch))
       return TRUE;
 
-   time_t log_files = newest_log_file_timestamp();
-   last_launch = MAX(log_files, last_launch);
-
    g_debug_update("time now %i (%s), delta: %i", now, ctime(&now), now-last_launch);
    if ((last_launch + (24*60*60*interval_days)) < now) {
       g_debug_update ("need to auto launch");
@@ -534,6 +531,26 @@ auto_launch_now (TrayApplet *ta)
    return FALSE;
 }
 
+static void
+launch_update_manager (GPid pid,
+                       gint status,
+                       gpointer user_data G_GNUC_UNUSED)
+{
+   g_autoptr(GError) error = NULL;
+
+   g_spawn_check_exit_status (status, &error);
+
+   /* list-oem-metapackages exited abnormally */
+   if (error != NULL)
+   {
+      g_printerr ("list-oem-metapackages exited abnormally: %s\n", error->message);
+      return;
+   }
+
+   g_spawn_command_line_async("nice ionice -c3 update-manager "
+                              "--no-update --no-focus-on-map", NULL);
+}
+
 gboolean
 update_check (TrayApplet *ta)
 {
@@ -643,8 +660,32 @@ update_check (TrayApplet *ta)
    tray_applet_ui_set_visible(ta, FALSE);
    if (auto_launch_now(ta))
    {
-      g_spawn_command_line_async("nice ionice -c3 update-manager "
-			         "--no-update --no-focus-on-map", NULL);
+      g_autoptr(GError) error = NULL;
+      g_autofree gchar *list_oem_metapackages = NULL;
+      GPid pid;
+      g_auto(GStrv) argv = g_new0(char *, 2);
+
+      argv[0] = g_build_filename (PACKAGE_LIB_DIR,
+                                  "update-notifier",
+                                  "list-oem-metapackages",
+                                  NULL);
+
+      g_spawn_async (NULL, /* working_directory */
+                     argv,
+                     NULL, /* envp */
+                     G_SPAWN_DO_NOT_REAP_CHILD,
+                     NULL, /* child_setup */
+                     NULL, /* user_data */
+                     &pid,
+                     &error);
+
+      if (error != NULL)
+      {
+         g_printerr("Failed to spawn ‘list-oem-metapackages’: %s\n", error->message);
+         return FALSE;
+      }
+
+      g_child_watch_add (pid, launch_update_manager, NULL);
    }
    return TRUE;
 
